@@ -33,7 +33,7 @@ use 5.005;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '0.9.1';
+$VERSION = '0.9.2';
 
 ### constructor
 
@@ -45,6 +45,7 @@ sub new {
     # TODO: Type checking.
     $self->{type} = $options{type} ? $options{type} : undef;
     $self->{href} = $options{href} ? $options{href} : undef;
+    $self->{class} = $options{class} ? $options{class} : undef;
     $self->{content} = $options{content} ? $options{content} : undef;
     $self->{children} = $options{children} ? $options{children} : undef;
     bless $self, $this;
@@ -67,6 +68,13 @@ sub href {
     return $this->{href};
 }
 
+sub class {
+    my $this = shift;
+
+    $this->{class} = shift if @_;
+    return $this->{class};
+}
+
 sub content {
     my $this = shift;
 
